#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# MOD Duo hardware tests
# Copyright (C) 2016-2020 Filipe Coelho <falktx@falktx.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# For a full copy of the GNU General Public License see the LICENSE file.

# ------------------------------------------------------------------------------------------------------------
# Imports

# usual stuff
import os
import sys

# additional stuff
from shutil import copy
from time import sleep

# from extra libraries
from serial import Serial

# local
from config import devInfo, testStream
from mod_protocol import CMD_GLCD_TEXT, CMD_GLCD_DRAW

# ------------------------------------------------------------------------------------------------------------
# check if running through mod-factory-deploy

testStream.from_pc = "--from-gui=yes" in sys.argv

# ------------------------------------------------------------------------------------------------------------
# collect all implemented tests

from test_gpio import tests as tests_gpio
from test_hmi import tests as tests_hmi
from test_audio import tests as tests_audio
from test_controlvoltage import tests as tests_controlvoltage
from test_midi import tests as tests_midi
from test_controlchain import tests as tests_controlchain
from test_network import tests as tests_network
from test_uid_eeprom import tests as tests_eeprom

# check for special test conditions
selftest_only_audio       = os.path.exists("/root/selftest-only-audio")
selftest_only_cv          = os.path.exists("/root/selftest-only-cv")
selftest_only_bottomboard = os.path.exists("/root/selftest-only-bottomboard")
selftest_only_hmi         = os.path.exists("/root/selftest-only-hmi")
selftest_only_restricted  = selftest_only_audio or selftest_only_cv or selftest_only_bottomboard or selftest_only_hmi

if selftest_only_restricted:
    testCollection = [ tests_gpio ]

    if selftest_only_audio:
        testCollection.append(tests_audio)
        testStream.hmi_available = not testStream.from_pc

    elif selftest_only_cv:
        testCollection.append(tests_controlvoltage)
        testStream.hmi_available = not testStream.from_pc

    elif selftest_only_bottomboard:
        testCollection.append(tests_audio)
        testCollection.append(tests_controlvoltage)
        testCollection.append(tests_midi)
        testCollection.append(tests_controlchain)
        testCollection.append(tests_network)
        testCollection.append(tests_eeprom)
        testStream.hmi_available = not testStream.from_pc

    elif selftest_only_hmi:
        testCollection.append(tests_hmi)

else:
    testCollection = [
        tests_gpio,
        tests_hmi,
        tests_audio,
        tests_controlvoltage,
        tests_midi,
        tests_controlchain,
        tests_network,
        tests_eeprom,
    ]

# ------------------------------------------------------------------------------------------------------------

def showText(msg1, msg2):
    if not testStream.hmi_available:
        print(msg1, msg2)
        return

    serial = Serial(devInfo.hmi_serial_port, devInfo.hmi_baud_rate, timeout=0.1)
    serial.flushInput()
    serial.flushOutput()

    bff = b"00"*1024

    serial.write("{} 0 0 0 8040".format(CMD_GLCD_DRAW).encode("utf-8") + bff + b"\0")
    while not serial.read_until(b"\0"):
        sleep(0.1)

    if devInfo.platform != "dwarf":
        serial.write("{} 1 0 0 8040".format(CMD_GLCD_DRAW).encode("utf-8") + bff + b"\0")
        while not serial.read_until(b"\0"):
            sleep(0.1)

    serial.write("{} 0 0 0 \"{}\"".format(CMD_GLCD_TEXT, msg1).encode("utf-8")+b"\0")
    while not serial.read_until(b"\0"):
        sleep(0.1)
    serial.write("{} 0 0 0 \"{}\"".format(CMD_GLCD_TEXT, msg1).encode("utf-8")+b"\0")
    while not serial.read_until(b"\0"):
        sleep(0.1)

    if devInfo.platform == "dwarf":
        serial.write("{} 0 0 32 \"{}\"".format(CMD_GLCD_TEXT, msg2).encode("utf-8")+b"\0")
        while not serial.read_until(b"\0"):
            sleep(0.1)
        serial.write("{} 0 0 32 \"{}\"".format(CMD_GLCD_TEXT, msg2).encode("utf-8")+b"\0")
        while not serial.read_until(b"\0"):
            sleep(0.1)
    else:
        serial.write("{} 1 0 0 \"{}\"".format(CMD_GLCD_TEXT, msg2).encode("utf-8")+b"\0")
        while not serial.read_until(b"\0"):
            sleep(0.1)
        serial.write("{} 1 0 0 \"{}\"".format(CMD_GLCD_TEXT, msg2).encode("utf-8")+b"\0")
        while not serial.read_until(b"\0"):
            sleep(0.1)

    serial.close()

# ------------------------------------------------------------------------------------------------------------
# Main

if __name__ == '__main__':
    from unittest import TestSuite, TextTestRunner, makeSuite

    if testStream.from_pc:
        print("self-test started")

    # stop services that might conflict with us
    servicesToStop = "browsepy controlchaind.service controlchaind.socket mod-ui jack2 jack-netmanager"

    if os.system("systemctl stop " + servicesToStop) != 0:
        print("Failed to stop conflicting services")
        sys.exit(1)

    os.system("killall -SIGKILL jackd 2>/dev/null")

    cases = []
    suite = TestSuite()

    for test in [tests for tests in testCollection for tests in tests]:
        suite.addTest(makeSuite(test))

    stream = open("/data/self-test/hw-test.log", 'w')
    runner = TextTestRunner(stream)
    testStream.setStreamFromRunner(runner)
    result = runner.run(suite)
    stream.close()

    ok = result.wasSuccessful()
    if ok:
        if os.path.exists("/root/need-self-test") and not selftest_only_restricted:
            os.remove("/root/need-self-test")
        if os.path.exists("/data/hw-test.log"):
            os.remove("/data/hw-test.log")

        os.system("sync")

        if not selftest_only_restricted:
            os.system("sleep 2")

        print("All ok!")
        showText("All tests passed", "{} is good!".format(devInfo.model_name))

    else:
        if selftest_only_restricted:
            with open("/data/self-test/hw-test.log", 'r') as fh:
                print(fh.read())
        else:
            copy("/data/self-test/hw-test.log", "/data/hw-test.log")
            os.system("sync")
            os.system("sleep 2")

        print("ERRORS happened")

        count = len(result.failures)+len(result.errors)
        showText("%i tests failed!" % count, "{} is bad!".format(devInfo.model_name))

    if testStream.from_pc:
        sys.exit(0 if ok else 1)
    else:
        sys.exit(0)
