#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# MOD Duo hardware tests
# Copyright (C) 2016-2020 Filipe Coelho <falktx@falktx.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# For a full copy of the GNU General Public License see the LICENSE file.

# ------------------------------------------------------------------------------------------------------------
# Imports

# usual stuff
import os
import sys

# additional stuff
from signal import SIGKILL
from subprocess import getoutput
from unittest import TestCase
from time import sleep

# from extra libraries
from serial import Serial

# local
from config import devInfo, testStream
from mod_protocol import CMD_RESPONSE, CMD_GLCD_TEXT, CMD_GLCD_DIALOG, CMD_GLCD_DRAW, CMD_DISP_BRIGHTNESS

# ------------------------------------------------------------------------------------------------------------

if os.path.exists("/usr/bin/mod-self-test.helper"):
    self_test_helper = "/usr/bin/mod-self-test.helper"
else:
    self_test_helper = os.path.join(sys.path[0], "mod-self-test.helper")

# ------------------------------------------------------------------------------------------------------------
# Common functions to all tests

def get_hardware_device(interface):
    if 'gpio' in interface:
        dev_match = interface.replace('gpio.', '').lower()
        dev_search_path = '/sys/devices/platform/gpio-sunxi/gpio'
        dev_path = '/sys/class/gpio/%s'
    elif 'uart' in interface:
        dev_match = 'tty'
        dev_search_path = '/sys/devices/platform/sunxi-' + interface + '/tty'
        dev_path = '/dev/%s'
    elif 'i2c' in interface:
        dev_match = 'i2c'
        dev_search_path = '/sys/devices/platform/sunxi-' + interface
        dev_path = '/sys/class/i2c-dev/%s'
    else:
        return None

    for dev in os.listdir(dev_search_path):
        if dev_match in dev:
            return dev_path % dev

    return None

def input_print_action(msg):
    return input("\n\033[32m%s\033[0m\n" % msg)

def kill_self():
    os.kill(os.getpid(), SIGKILL)

def range_list(value):
    return list(range(value+1)) + list(reversed(range(value)))

# ------------------------------------------------------------------------------------------------------------

class ExtendedTestCase(TestCase):
    def assertCommand(self, cmd):
        print(cmd)
        self.assertEqual(os.system(cmd+" &>/dev/null"), 0)

    def assertCommandOutput(self, cmd, expectedOutput):
        self.assertEqual(getoutput(cmd).strip(), expectedOutput.strip())

# ------------------------------------------------------------------------------------------------------------
# pylint: disable=duplicate-code

class ControllerTestCase(ExtendedTestCase):
    def setUp(self):
        if not testStream.hmi_available:
            return
        self.serial = Serial(devInfo.hmi_serial_port, devInfo.hmi_baud_rate, timeout=0.1)
        self.serial.flushInput()
        self.serial.flushOutput()

    def tearDown(self):
        if not testStream.hmi_available:
            return
        self.serial.close()

    def setMaxDisplayBrightness(self):
        if not testStream.hmi_available:
            return
        self.writeCustomMessage("{} 4".format(CMD_DISP_BRIGHTNESS), False)
        self.writeCustomMessage("{} 4".format(CMD_DISP_BRIGHTNESS), False)

    def clearDisplay1(self):
        if not testStream.hmi_available:
            return
        bff = b"00"*1024
        self.writeCustomMessage("{} 0 0 0 8040".format(CMD_GLCD_DRAW).encode("utf-8") + bff + b"\0")
        self.writeCustomMessage("{} 0 0 0 8040".format(CMD_GLCD_DRAW).encode("utf-8") + bff + b"\0")

    def clearDisplay2(self):
        if not testStream.hmi_available:
            return
        bff = b"00"*1024
        if devInfo.platform == "dwarf":
            self.writeCustomMessage("{} 0 0 0 8040".format(CMD_GLCD_DRAW).encode("utf-8") + bff + b"\0")
            self.writeCustomMessage("{} 0 0 0 8040".format(CMD_GLCD_DRAW).encode("utf-8") + bff + b"\0")
        else:
            self.writeCustomMessage("{} 1 0 0 8040".format(CMD_GLCD_DRAW).encode("utf-8") + bff + b"\0")
            self.writeCustomMessage("{} 1 0 0 8040".format(CMD_GLCD_DRAW).encode("utf-8") + bff + b"\0")

    def waitForGuiPress(self):
        b = input()
        print("got input", b)
        return b == "1"

    def showText(self, msg, disp=1, x=0, y=0):
        if not testStream.hmi_available:
            print(msg)
            return

        print("TEXT:", msg)
        if devInfo.platform == "dwarf":
            if disp == 1:
                y = 32
            disp = 0

        self.serial.write("{} {} {} {} \"{}\"".format(CMD_GLCD_TEXT, disp, x, y, msg).encode("utf-8")+b"\0")

        respcheck = "{} ".format(CMD_RESPONSE)
        while True:
            resp = self.serial.read_until(b"\0")
            if not resp:
                continue
            resp = resp[:-1].decode("utf-8", errors="ignore")
            if resp[:len(respcheck)] != respcheck:
                continue
            return

    def showYesNoDialog(self, msg, checkResponse=True):
        if not testStream.hmi_available:
            print("GUIDIALOG:", msg.replace(".\n\n",". ").replace("\n\n","; ").replace("\n"," "))
            return self.waitForGuiPress()

        print("DIALOG:", msg)
        self.serial.write("{} \"{}\"".format(CMD_GLCD_DIALOG, msg).encode("utf-8")+b"\0")

        respcheck = "{} ".format(CMD_RESPONSE)
        respreturn = "{} 0".format(CMD_RESPONSE)
        while True:
            resp = self.serial.read_until(b"\0")
            if not resp:
                continue
            resp = resp[:-1].decode("utf-8", errors="ignore")
            if resp[:len(respcheck)] != respcheck:
                continue
            if checkResponse:
                self.assertEqual(resp, respreturn)
            self.clearDisplay2()
            return bool(resp == respreturn)

    # NOTE only used for HMI stuff
    def writeCustomMessage(self, msg, checkResponse=True):
        #TODO THIS IS A TEMPORARY FIX
        sleep(0.03)

        if isinstance(msg, bytes):
            self.serial.write(msg)
        else:
            self.serial.write(msg.encode("utf-8")+b"\0")

        respcheck = "{} ".format(CMD_RESPONSE)
        respreturn = "{} 0".format(CMD_RESPONSE)
        while True:
            resp = self.serial.read_until(b"\0")
            if not resp:
                continue
            resp = resp[:-1].decode("utf-8", errors="ignore")
            if resp[:len(respcheck)] != respcheck:
                continue
            if checkResponse:
                self.assertEqual(resp, respreturn)
            return bool(resp == respreturn)

# ------------------------------------------------------------------------------------------------------------
