#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# MOD Duo hardware tests
# Copyright (C) 2016-2020 Filipe Coelho <falktx@falktx.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# For a full copy of the GNU General Public License see the LICENSE file.

# ------------------------------------------------------------------------------------------------------------
# Imports

# additional stuff
from time import sleep

# local
from config import devInfo, testStream
from mod_protocol import (
    CMD_PING,
    CMD_LED,
    CMD_GLCD_DRAW,
    CMD_RESPONSE,
    CMD_CONTROL_ADD,
    CMD_CONTROL_REMOVE,
    CMD_CONTROL_SET,
    CMD_PEDALBOARD_CLEAR,
    CMD_DUO_CONTROL_NEXT,
    CMD_SELFTEST_ENCODER_CLICKED,
    CMD_SELFTEST_ENCODER_LEFT,
    CMD_SELFTEST_ENCODER_RIGHT,
    CMD_SELFTEST_BUTTON_CLICKED,
    CMD_SELFTEST_CHECK_CALIBRATION,
    CMD_SELFTEST_SAVE_POT_CALIBRATION,
    CMD_SELFTEST_SKIP_CONTROL,
    CMD_SELFTEST_SKIP_CONTROL_ENABLE,
    CMD_DISP_BRIGHTNESS,
)
from test_abstract import ControllerTestCase, kill_self, self_test_helper

# ------------------------------------------------------------------------------------------------------------

class TestHmiSetup(ControllerTestCase):
    def test_setup(self):
        # test if HMI is up
        if not self.ping():
            print("HMI not online, failed to respond to ping")
            kill_self()
            return

        self.clearDisplay1()

        # test if operator knows what to do (try 2 times)
        for i in range(2):
            try:
                msg = ("Self test mode.\n"
                       "Disconnect all cables\n"
                       "and select 'YES' to continue")
                if i != 0:
                    msg += "\nLast attempt."
                self.clearDisplay2()
                self.showYesNoDialog(msg)
                break
            # pylint: disable=bare-except
            except:
                if i == 0:
                    continue

            self.clearDisplay1()
            self.showText("ERROR", 0)
            self.showText("ERROR", 0)
            self.showText("Must select 'YES'", 1)
            self.showText("Must select 'YES'", 1)
            kill_self()

    def ping(self):
        self.serial.write(CMD_PING.encode("utf-8")+b"\0")

        retries = 200
        respcheck = "{} ".format(CMD_RESPONSE).encode("utf-8")
        while retries > 0:
            resp = self.serial.readall()
            if not resp:
                retries -= 1
                continue
            resps = resp.split(b"\0")
            for resp in resps:
                if not resp:
                    continue
                if resp[:len(CMD_RESPONSE)+1] != respcheck:
                    continue
                return bool(resp == "{} 0".format(CMD_RESPONSE).encode("utf-8"))

class TestHmiScreen(ControllerTestCase):
    def test_pixels(self):
        for i in reversed(range(1, 2+1)):
            self.clearDisplay2()
            self.showText("Checking display pixels in %i..." %i, 1, 0, 0)
            self.showText("Checking display pixels in %i..." %i, 1, 0, 0) # NOTE called twice just in case
            sleep(0.9)

        bff = b"FF"*1024
        self.writeCustomMessage("{} 0 0 0 8040".format(CMD_GLCD_DRAW).encode("utf-8") + bff + b"\0")
        if devInfo.platform != "dwarf":
            self.writeCustomMessage("{} 1 0 0 8040".format(CMD_GLCD_DRAW).encode("utf-8") + bff + b"\0")

        sleep(2)
        self.clearDisplay1()
        self.clearDisplay2()
        self.showYesNoDialog("Are the pixels ok?")

    def test_brightness(self):
        for i in reversed(range(1, 2+1)):
            self.clearDisplay2()
            self.showText("Checking display brightness in %i..." %i, 1, 0, 0)
            self.showText("Checking display brightness in %i..." %i, 1, 0, 0) # NOTE called twice just in case
            sleep(0.9)

        self.clearDisplay1()
        self.clearDisplay2()

        for i in range(0,4):
            self.writeCustomMessage("{} {}".format(CMD_DISP_BRIGHTNESS, i))
            sleep(0.6)

        self.showYesNoDialog("are all 4 levels of brightness\nworking?\n\n( off, low, mid, high )")

class TestHmiLeds(ControllerTestCase):
    # pylint: disable=bad-whitespace
    kAudioLedColorRed   = 1
    kAudioLedColorGreen = 2
    kAudioLedColorBlue  = 0
    kAudioLedColorOff   = 3

    kCtrlLedColorRed     = (CMD_LED, 0x64, 0x00, 0x00)
    kCtrlLedColorGreen   = (CMD_LED, 0x00, 0x64, 0x00)
    kCtrlLedColorBlue    = (CMD_LED, 0x00, 0x00, 0x64)
    kCtrlLedColorYellow  = (CMD_LED, 0x64, 0x64, 0x00)
    kCtrlLedColorMagenta = (CMD_LED, 0x64, 0x00, 0x64)
    kCtrlLedColorCyan    = (CMD_LED, 0x00, 0x64, 0x64)
    kCtrlLedColorWhite   = (CMD_LED, 0x64, 0x64, 0x64)
    kCtrlLedColorOff     = (CMD_LED, 0x00, 0x00, 0x00)
    # pylint: enable=bad-whitespace

    def setUp(self):
        ControllerTestCase.setUp(self)

        with open("%s/direction" % devInfo.pca9685_gpio_path, 'w') as fh:
            fh.write("out\n")
        with open("%s/value" % devInfo.pca9685_gpio_path, 'w') as fh:
            fh.write("0\n")

    def test_leds_1_red(self):
        self.writeCustomMessage("{} 0 {} {} {}".format(*self.kCtrlLedColorRed))
        self.writeCustomMessage("{} 1 {} {} {}".format(*self.kCtrlLedColorRed))

        if devInfo.platform in ("duox", "dwarf"):
            self.writeCustomMessage("{} 2 {} {} {}".format(*self.kCtrlLedColorRed))
            self.writeCustomMessage("{} 3 {} {} {}".format(*self.kCtrlLedColorRed))
            self.writeCustomMessage("{} 4 {} {} {}".format(*self.kCtrlLedColorRed))
            self.writeCustomMessage("{} 5 {} {} {}".format(*self.kCtrlLedColorRed))
            self.writeCustomMessage("{} 6 {} {} {}".format(*self.kCtrlLedColorRed))

        try:
            self.assertCommand("{} leds {}".format(self_test_helper, self.kAudioLedColorRed))
        except AssertionError as e:
            testStream.print("Communication with LEDs failed")
            raise e

        self.showYesNoDialog("LEDs are 'red'")

    def test_leds_2_green(self):
        self.writeCustomMessage("{} 0 {} {} {}".format(*self.kCtrlLedColorGreen))
        self.writeCustomMessage("{} 1 {} {} {}".format(*self.kCtrlLedColorGreen))

        if devInfo.platform in ("duox", "dwarf"):
            self.writeCustomMessage("{} 2 {} {} {}".format(*self.kCtrlLedColorGreen))
            self.writeCustomMessage("{} 3 {} {} {}".format(*self.kCtrlLedColorGreen))
            self.writeCustomMessage("{} 4 {} {} {}".format(*self.kCtrlLedColorGreen))
            self.writeCustomMessage("{} 5 {} {} {}".format(*self.kCtrlLedColorGreen))
            self.writeCustomMessage("{} 6 {} {} {}".format(*self.kCtrlLedColorGreen))

        try:
            self.assertCommand("{} leds {}".format(self_test_helper, self.kAudioLedColorGreen))
        except AssertionError as e:
            testStream.print("Communication with LEDs failed")
            raise e

        self.showYesNoDialog("LEDs are 'green'")

    def test_leds_3_blue(self):
        self.writeCustomMessage("{} 0 {} {} {}".format(*self.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 1 {} {} {}".format(*self.kCtrlLedColorBlue))

        if devInfo.platform in ("duox", "dwarf"):
            self.writeCustomMessage("{} 2 {} {} {}".format(*self.kCtrlLedColorBlue))
            self.writeCustomMessage("{} 3 {} {} {}".format(*self.kCtrlLedColorBlue))
            self.writeCustomMessage("{} 4 {} {} {}".format(*self.kCtrlLedColorBlue))
            self.writeCustomMessage("{} 5 {} {} {}".format(*self.kCtrlLedColorBlue))
            self.writeCustomMessage("{} 6 {} {} {}".format(*self.kCtrlLedColorBlue))

        try:
            self.assertCommand("%s leds %i" % (self_test_helper, self.kAudioLedColorBlue))
        except AssertionError as e:
            testStream.print("Communication with LEDs failed")
            raise e

        self.showYesNoDialog("LEDs are 'blue'")

    def test_leds_4_off(self):
        self.writeCustomMessage("{} 0 {} {} {}".format(*self.kCtrlLedColorOff))
        self.writeCustomMessage("{} 1 {} {} {}".format(*self.kCtrlLedColorOff))

        if devInfo.platform in ("duox", "dwarf"):
            self.writeCustomMessage("{} 2 {} {} {}".format(*self.kCtrlLedColorOff))
            self.writeCustomMessage("{} 3 {} {} {}".format(*self.kCtrlLedColorOff))
            self.writeCustomMessage("{} 4 {} {} {}".format(*self.kCtrlLedColorOff))
            self.writeCustomMessage("{} 5 {} {} {}".format(*self.kCtrlLedColorOff))
            self.writeCustomMessage("{} 6 {} {} {}".format(*self.kCtrlLedColorOff))

        try:
            self.assertCommand("%s leds %i" % (self_test_helper, self.kAudioLedColorOff))
        except AssertionError as e:
            testStream.print("Communication with LEDs failed")
            raise e

        self.showYesNoDialog("LEDs are off")

class TestHmiAddressings(ControllerTestCase):
    def showTextAndWaitForMessage(self, disp, msg, waitMsg, timeoutMsg):
        if devInfo.platform == "dwarf":
            disp = 0

        self.showText(msg, disp)

        # NOTE send text twice just in case
        self.showText(msg, disp)

        while True:
            resp = self.serial.read_until(b"\0")
            if not resp:
                continue
            resp = resp[:-1].decode("utf-8", errors="ignore")
            if resp.startswith(waitMsg):
                if waitMsg.split(None, 1)[0] in (CMD_DUO_CONTROL_NEXT,):
                    self.serial.write("{} 0".format(CMD_RESPONSE).encode("utf-8")+b"\0")
                return True
            if resp.startswith(timeoutMsg):
                self.serial.write("{} 0".format(CMD_RESPONSE).encode("utf-8")+b"\0")
                return False
            self.serial.write("{} 0".format(CMD_RESPONSE).encode("utf-8")+b"\0")

    def writeCustomMessageCheckResponse(self, msg, checkResponse=True):
        if isinstance(msg, bytes):
            self.serial.write(msg)
        else:
            self.serial.write(msg.encode("utf-8")+b"\0")

        check_resp_1 = "{} 1".format(CMD_RESPONSE)
        check_resp_0 = "{} 0".format(CMD_RESPONSE)
        while True:
            resp = self.serial.read_until(b"\0")
            if not resp:
                continue
            resp = resp[:-1].decode("utf-8", errors="ignore")
            if checkResponse:
                if resp == check_resp_1:
                    return True
                if resp == check_resp_0:
                    return False

    def setUp(self):
        ControllerTestCase.setUp(self)

        if devInfo.platform == "duo":
            self.resp_knob_clicked_msg = CMD_DUO_CONTROL_NEXT
        else:
            self.resp_knob_clicked_msg = CMD_SELFTEST_ENCODER_CLICKED

    #TODO split device selftest up over different files
    def test_01_mdx_tiny(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 4 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        self.showTextAndWaitForMessage(1, "Press the red button",
                                       "{} 4".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage("{} 4 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorGreen))
        self.showTextAndWaitForMessage(1, "Press the green button",
                                       "{} 5".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.showTextAndWaitForMessage(1, "Press the blue button",
                                       "{} 6".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.clearDisplay1()
        self.clearDisplay2()

        #enable the control skip button
        self.writeCustomMessage("{}".format(CMD_SELFTEST_SKIP_CONTROL_ENABLE))

    def test_02_mdx_left_encoder(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()

        sleep(0.1)

        g = self.showTextAndWaitForMessage(0, "Press top left KNOB down \n \nif this is not working press   the red button",
                                           "{} 0".format(self.resp_knob_clicked_msg), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Left encoder broken does not click")
            raise e

        self.clearDisplay1()
        self.clearDisplay2()

        sleep(0.1)

        g = self.showTextAndWaitForMessage(0, "Move top left KNOB left\n \nif this is not working press   the red button",
                                           "{} 0".format(CMD_SELFTEST_ENCODER_LEFT), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Left encoder broken does not turn left")
            raise e

        self.clearDisplay1()
        self.clearDisplay2()

        sleep(0.1)

        g = self.showTextAndWaitForMessage(0, "Move top left KNOB right\n \nif this is not working press   the red button",
                                           "{} 0".format(CMD_SELFTEST_ENCODER_RIGHT), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Left encoder broken does not turn right")
            raise e

    def test_03_mdx_left_most_button(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage("{} 0 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))

        sleep(0.1)

        g = self.showTextAndWaitForMessage(0, "Press the left-most button\n \nif this is not working press   the red button",
                                           "{} 0".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Left-most button broken")
            raise e

        self.writeCustomMessage("{} 1 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))

    def test_04_mdx_left_middle_button(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()

        self.writeCustomMessage("{} 0 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 1 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))

        sleep(0.1)

        g = self.showTextAndWaitForMessage(0, "Press the middle-left button\n \nif this is not working press   the red button",
                                           "{} 1".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Left-middle button broken")
            raise e

        self.writeCustomMessage("{} 1 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))

    def test_05_mdx_pot_1(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()

        #pot 1 test
        self.writeCustomMessage("{} 6 \"POT1\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay2()
        self.writeCustomMessage("{} 0 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.showTextAndWaitForMessage(0, "Move Pot 1 all the way left\nand press the blue button",
                                       "{} 0".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 0 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 0 0".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.clearDisplay1()
        self.writeCustomMessage("{} 1 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 6 \"POT1\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay2()
        self.showTextAndWaitForMessage(0, "Move Pot 1 all the way right\nand press the blue button",
                                       "{} 1".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 1 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 1 0".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.writeCustomMessage("{} 6".format(CMD_CONTROL_REMOVE))

        g = self.writeCustomMessageCheckResponse("{} 0".format(CMD_SELFTEST_CHECK_CALIBRATION))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Pot 1 calibration not valid")
            raise e

    def test_06_mdx_pot_2(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()

        #pot 2 test
        self.writeCustomMessage("{} 7 \"POT2\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay2()
        self.writeCustomMessage("{} 0 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.showTextAndWaitForMessage(0, "Move Pot 2 all the way left\nand press the blue button",
                                       "{} 0".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 0 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 0 1".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.clearDisplay1()
        self.writeCustomMessage("{} 1 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 7 \"POT2\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay2()
        self.showTextAndWaitForMessage(0, "Move Pot 2 all the way right\nand press the blue button",
                                       "{} 1".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 1 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 1 1".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.writeCustomMessage("{} 7".format(CMD_CONTROL_REMOVE))

        g = self.writeCustomMessageCheckResponse("{} 1".format(CMD_SELFTEST_CHECK_CALIBRATION))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Pot 2 calibration not valid")
            raise e

    def test_07_mdx_pot_3(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()

        #pot 3 test
        self.writeCustomMessage("{} 8 \"POT3\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay2()
        self.writeCustomMessage("{} 0 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.showTextAndWaitForMessage(0, "Move Pot 3 all the way left\nand press the blue button",
                                       "{} 0".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 0 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 0 2".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.clearDisplay1()
        self.writeCustomMessage("{} 1 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 8 \"POT3\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay2()
        self.showTextAndWaitForMessage(0, "Move Pot 3 all the way right\nand press the blue button",
                                       "{} 1".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 1 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 1 2".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.writeCustomMessage("{} 8".format(CMD_CONTROL_REMOVE))

        g = self.writeCustomMessageCheckResponse("{} 2".format(CMD_SELFTEST_CHECK_CALIBRATION))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Pot 3 calibration not valid")
            raise e

    def test_08_mdx_pot_4(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()

        #pot 4 test
        self.writeCustomMessage("{} 9 \"POT4\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay2()
        self.writeCustomMessage("{} 0 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.showTextAndWaitForMessage(0, "Move Pot 4 all the way left\nand press the blue button",
                                       "{} 0".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 0 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 0 3".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.clearDisplay1()
        self.writeCustomMessage("{} 1 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 9 \"POT4\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay2()
        self.showTextAndWaitForMessage(0, "Move Pot 4 all the way right\nand press the blue button",
                                       "{} 1".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 1 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 1 3".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.writeCustomMessage("{} 9".format(CMD_CONTROL_REMOVE))

        g = self.writeCustomMessageCheckResponse("{} 3".format(CMD_SELFTEST_CHECK_CALIBRATION))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Pot 4 calibration not valid")
            raise e

    def test_09_mdx_right_encoder(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()

        sleep(0.1)

        g = self.showTextAndWaitForMessage(1, "Press top right KNOB down \n \nif this is not working press   the red button",
                                           "{} 1".format(self.resp_knob_clicked_msg), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Right encoder broken does not click")
            raise e

        self.clearDisplay1()
        self.clearDisplay2()

        sleep(0.1)

        g = self.showTextAndWaitForMessage(1, "Move top right KNOB left\n \nif this is not working press   the red button",
                                           "{} 1".format(CMD_SELFTEST_ENCODER_LEFT), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Right encoder broken does not turn left")
            raise e

        self.clearDisplay1()
        self.clearDisplay2()

        g = self.showTextAndWaitForMessage(1, "Move top right KNOB right\n \nif this is not working press   the red button",
                                           "{} 1".format(CMD_SELFTEST_ENCODER_RIGHT), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        sleep(0.1)

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Right encoder broken does not turn right")
            raise e

    def test_10_mdx_right_most_button(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage("{} 2 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))

        sleep(0.1)

        g = self.showTextAndWaitForMessage(1, "Press the right-middle button\n \nif this is not working press   the red button",
                                           "{} 2".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Right-middle button broken")
            raise e

        self.writeCustomMessage("{} 2 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))

    def test_11_mdx_right_middle_button(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()

        self.writeCustomMessage("{} 2 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 3 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))

        sleep(0.1)

        g = self.showTextAndWaitForMessage(1, "Press the right-most button\n \nif this is not working press   the red button",
                                           "{} 3".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Right-most button broken")
            raise e

        self.writeCustomMessage("{} 3 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))

    def test_12_mdx_pot_5(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()

        #pot 5 test
        self.writeCustomMessage("{} 10 \"POT5\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay1()
        self.writeCustomMessage("{} 2 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.showTextAndWaitForMessage(1, "Move Pot 5 all the way left\nand press the blue button",
                                       "{} 2".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 2 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 0 4".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.clearDisplay2()
        self.writeCustomMessage("{} 3 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 10 \"POT5\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay1()
        self.showTextAndWaitForMessage(1, "Move Pot 5 all the way right\nand press the blue button",
                                       "{} 3".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 3 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 1 4".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.writeCustomMessage("{} 10".format(CMD_CONTROL_REMOVE))

        g = self.writeCustomMessageCheckResponse("{} 4".format(CMD_SELFTEST_CHECK_CALIBRATION))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Pot 5 calibration not valid")
            raise e

    def test_13_mdx_pot_6(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()

        #pot 6 test
        self.writeCustomMessage("{} 11 \"POT6\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay1()
        self.writeCustomMessage("{} 2 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.showTextAndWaitForMessage(1, "Move Pot 6 all the way left\nand press the blue button",
                                       "{} 2".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 2 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 0 5".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.clearDisplay2()
        self.writeCustomMessage("{} 3 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 11 \"POT6\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay1()
        self.showTextAndWaitForMessage(1, "Move Pot 6 all the way right\nand press the blue button",
                                       "{} 3".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 3 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 1 5".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.writeCustomMessage("{} 11".format(CMD_CONTROL_REMOVE))

        g = self.writeCustomMessageCheckResponse("{} 5".format(CMD_SELFTEST_CHECK_CALIBRATION))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Pot 6 calibration not valid")
            raise e

    def test_14_mdx_pot_7(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()

        #pot 7 test
        self.writeCustomMessage("{} 12 \"POT7\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay1()
        self.writeCustomMessage("{} 2 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.showTextAndWaitForMessage(1, "Move Pot 7 all the way left\nand press the blue button",
                                       "{} 2".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 2 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 0 6".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.clearDisplay2()
        self.writeCustomMessage("{} 3 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 12 \"POT7\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay1()
        self.showTextAndWaitForMessage(1, "Move Pot 7 all the way right\nand press the blue button",
                                       "{} 3".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 3 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 1 6".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.writeCustomMessage("{} 12".format(CMD_CONTROL_REMOVE))

        g = self.writeCustomMessageCheckResponse("{} 6".format(CMD_SELFTEST_CHECK_CALIBRATION))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Pot 7 calibration not valid")
            raise e

    def test_15_mdx_pot_8(self):
        if devInfo.platform != "duox":
            return

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()

        #pot 8 test
        self.writeCustomMessage("{} 13 \"POT8\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay1()
        self.writeCustomMessage("{} 2 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.showTextAndWaitForMessage(1, "Move Pot 8 all the way left\nand press the blue button",
                                       "{} 2".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 2 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 0 7".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.clearDisplay2()
        self.writeCustomMessage("{} 3 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 13 \"POT8\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay1()
        self.showTextAndWaitForMessage(1, "Move Pot 8 all the way right\nand press the blue button",
                                       "{} 3".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.writeCustomMessage("{} 3 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 1 7".format(CMD_SELFTEST_SAVE_POT_CALIBRATION))
        self.writeCustomMessage("{} 13".format(CMD_CONTROL_REMOVE))

        g = self.writeCustomMessageCheckResponse("{} 7".format(CMD_SELFTEST_CHECK_CALIBRATION))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Pot 7 calibration not valid")
            raise e

    def test_16_mdw_shift(self):
        if devInfo.platform != "dwarf":
            return

        self.clearDisplay1()
        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorGreen))
        self.showTextAndWaitForMessage(1, "Press the green button",
                                       "{} 6".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))
        self.clearDisplay1()

        #enable the control skip button
        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        self.writeCustomMessage("{}".format(CMD_SELFTEST_SKIP_CONTROL_ENABLE))

    def test_17_mdw_left_button(self):
        if devInfo.platform != "dwarf":
            return

        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage("{} 3 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))

        sleep(0.1)

        g = self.showTextAndWaitForMessage(1, "Press the left button (blue)\n \nif this is not working press   the red button",
                                           "{} 3".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Left button broken")
            raise e

    def test_18_mdw_middle_button(self):
        if devInfo.platform != "dwarf":
            return

        self.writeCustomMessage("{} 3 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))

        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage("{} 4 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))

        sleep(0.1)

        g = self.showTextAndWaitForMessage(1, "Press the middle button (blue)\n \nif this is not working press   the red button",
                                           "{} 4".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Middle button broken")
            raise e

    def test_19_mdw_right_button(self):
        if devInfo.platform != "dwarf":
            return

        self.writeCustomMessage("{} 4 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))

        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))

        sleep(0.1)

        g = self.showTextAndWaitForMessage(1, "Press the right button (blue)\n \nif this is not working press   the red button",
                                           "{} 5".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Right button broken")
            raise e

    def test_20_mdw_left_encoder(self):
        if devInfo.platform != "dwarf":
            return

        self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))

        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        self.clearDisplay1()
        self.clearDisplay2()

        sleep(0.1)

        g = self.showTextAndWaitForMessage(1, "Press left encoder down \n \nif this is not working press   the red button",
                                           "{} 0".format(self.resp_knob_clicked_msg), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Left encoder broken does not click")
            raise e

        self.clearDisplay1()
        self.clearDisplay2()

        sleep(0.1)
        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        g = self.showTextAndWaitForMessage(1, "Move left encoder left\n \nif this is not working press   the red button",
                                           "{} 0".format(CMD_SELFTEST_ENCODER_LEFT), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Left encoder broken does not turn left")
            raise e

        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        g = self.showTextAndWaitForMessage(1, "Move left encoder right\n \nif this is not working press   the red button",
                                           "{} 0".format(CMD_SELFTEST_ENCODER_RIGHT), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        sleep(0.1)

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Left encoder broken does not turn right")
            raise e

    def test_21_mdw_middle_encoder(self):
        if devInfo.platform != "dwarf":
            return

        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        self.clearDisplay1()
        self.clearDisplay2()

        sleep(0.1)

        g = self.showTextAndWaitForMessage(1, "Press middle encoder down \n \nif this is not working press   the red button",
                                           "{} 1".format(self.resp_knob_clicked_msg), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Middle encoder broken does not click")
            raise e

        self.clearDisplay1()
        self.clearDisplay2()

        sleep(0.1)
        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        g = self.showTextAndWaitForMessage(1, "Move middle encoder left\n \nif this is not working press   the red button",
                                           "{} 1".format(CMD_SELFTEST_ENCODER_LEFT), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Middle encoder broken does not turn left")
            raise e

        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        g = self.showTextAndWaitForMessage(1, "Move middle encoder right\n \nif this is not working press   the red button",
                                           "{} 1".format(CMD_SELFTEST_ENCODER_RIGHT), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        sleep(0.1)

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Middle encoder broken does not turn right")
            raise e

    def test_22_mdw_right_encoder(self):
        if devInfo.platform != "dwarf":
            return

        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        self.clearDisplay1()
        self.clearDisplay2()

        sleep(0.1)

        g = self.showTextAndWaitForMessage(1, "Press right encoder down \n \nif this is not working press   the red button",
                                           "{} 2".format(self.resp_knob_clicked_msg), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Right encoder broken does not click")
            raise e

        self.clearDisplay1()
        self.clearDisplay2()

        sleep(0.1)
        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        g = self.showTextAndWaitForMessage(1, "Move right encoder left\n \nif this is not working press   the red button",
                                           "{} 2".format(CMD_SELFTEST_ENCODER_LEFT), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Right encoder broken does not turn left")
            raise e

        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        g = self.showTextAndWaitForMessage(1, "Move right encoder right\n \nif this is not working press   the red button",
                                           "{} 2".format(CMD_SELFTEST_ENCODER_RIGHT), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        sleep(0.1)

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Right encoder broken does not turn right")
            raise e

    def test_23_mdw_left_foot(self):
        if devInfo.platform != "dwarf":
            return

        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage("{} 2 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))

        sleep(0.1)

        g = self.showTextAndWaitForMessage(1, "Press the left footswitch\n \nif this is not working press   the red button",
                                           "{} 2".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Left footswitch broken")
            raise e

    def test_24_mdw_middle_foot(self):
        if devInfo.platform != "dwarf":
            return

        self.writeCustomMessage("{} 2 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))

        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage("{} 0 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))

        sleep(0.1)

        g = self.showTextAndWaitForMessage(1, "Press the middle footswitch\n \nif this is not working press   the red button",
                                           "{} 0".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Middle footswitch broken")
            raise e

    def test_25_mdw_right_foot(self):
        if devInfo.platform != "dwarf":
            return

        self.writeCustomMessage("{} 0 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))

        self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorRed))
        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage("{} 1 {} {} {}".format(*TestHmiLeds.kCtrlLedColorBlue))

        sleep(0.1)

        g = self.showTextAndWaitForMessage(1, "Press the right footswitch\n \nif this is not working press   the red button",
                                           "{} 1".format(CMD_SELFTEST_BUTTON_CLICKED), "{}".format(CMD_SELFTEST_SKIP_CONTROL))

        try:
            self.assertEqual(g, True)
        except AssertionError as e:
            testStream.print("Right footswitch broken")
            raise e

    def test_26_modduo(self):
        if devInfo.platform != "duo":
            return

        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)

        self.writeCustomMessage("{} 0 \"endless knob\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay2()
        self.showTextAndWaitForMessage(1, "Press left KNOB down",
                                       "{} 0".format(self.resp_knob_clicked_msg), "{} 0".format(self.resp_knob_clicked_msg))
        self.clearDisplay2()
        self.showTextAndWaitForMessage(1, "Move left KNOB left",
                                       "{} 0 -1.0".format(CMD_CONTROL_SET), "{} 0 -1.0".format(CMD_CONTROL_SET))
        self.clearDisplay2()
        self.showTextAndWaitForMessage(1, "Move left KNOB right",
                                       "{} 0 1.0".format(CMD_CONTROL_SET), "{} 0 1.0".format(CMD_CONTROL_SET))
        self.writeCustomMessage("{} 0".format(CMD_CONTROL_REMOVE))

        self.writeCustomMessage("{} 2 \"FOOTSWITCH\" 32 none 1 1 0 1 0".format(CMD_CONTROL_ADD))
        self.clearDisplay2()
        self.showTextAndWaitForMessage(1, "Press left FOOTSWITCH",
                                       "{} 2 0".format(CMD_CONTROL_SET), "{} 2 0".format(CMD_CONTROL_SET))
        self.writeCustomMessage("{} 2".format(CMD_CONTROL_REMOVE))

        self.clearDisplay1()
        self.clearDisplay2()
        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)

        self.writeCustomMessage("{} 1 \"endless knob\" 0 none 0.0 1.0 -1.0 5 0".format(CMD_CONTROL_ADD))
        self.clearDisplay1()
        self.showTextAndWaitForMessage(0, "Press right KNOB down ", "{} 1".format(self.resp_knob_clicked_msg), "{} 1".format(self.resp_knob_clicked_msg))
        self.clearDisplay1()
        self.showTextAndWaitForMessage(0, "Move right KNOB left",
                                       "{} 1 -1.0".format(CMD_CONTROL_SET), "{} 1 -1.0".format(CMD_CONTROL_SET))
        self.clearDisplay1()
        self.showTextAndWaitForMessage(0, "Move right KNOB right",
                                       "{} 1 1.0".format(CMD_CONTROL_SET), "{} 1 1.0".format(CMD_CONTROL_SET))
        self.writeCustomMessage("{} 1".format(CMD_CONTROL_REMOVE))

        self.writeCustomMessage("{} 3 \"FOOTSWITCH\" 32 none 1 1 0 1 0".format(CMD_CONTROL_ADD))
        self.clearDisplay1()
        self.showTextAndWaitForMessage(0, "Press right FOOTSWITCH",
                                       "{} 3 0".format(CMD_CONTROL_SET), "{} 3 0".format(CMD_CONTROL_SET))
        self.writeCustomMessage("{} 3".format(CMD_CONTROL_REMOVE))


    def test_27_cleanup(self):
        self.writeCustomMessage("{} 0 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
        self.writeCustomMessage("{} 1 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))

        if devInfo.platform in ("duox", "dwarf"):
            self.writeCustomMessage("{} 2 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
            self.writeCustomMessage("{} 3 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
            self.writeCustomMessage("{} 4 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
            self.writeCustomMessage("{} 5 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))
            self.writeCustomMessage("{} 6 {} {} {}".format(*TestHmiLeds.kCtrlLedColorOff))

        self.writeCustomMessage(CMD_PEDALBOARD_CLEAR)
        self.clearDisplay1()
        self.clearDisplay2()
        self.showText("Starting audio tests", 0)
        self.showText("Starting audio tests", 0)
        self.showText("Please wait...", 1)
        self.showText("Please wait...", 1)

# ------------------------------------------------------------------------------------------------------------

tests = [
    TestHmiSetup,
    TestHmiScreen,
    TestHmiLeds,
    TestHmiAddressings
]

# ------------------------------------------------------------------------------------------------------------
